# This file is part of PyOP2
#
# PyOP2 is Copyright (c) 2012, Imperial College London and
# others. Please see the AUTHORS file in the main source directory for
# a full list of copyright holders.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The name of Imperial College London or that of other
#       contributors may not be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTERS
# ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.

import pytest
from pyop2.profiling import tic, toc, get_timers, reset_timers, Timer


class TestProfiling:

    """Profiling tests."""

    def test_create(self):
        tic('create')
        toc('create')
        assert 'create' in get_timers().keys()

    def test_elapsed_nonstarted_fails(self):
        t = Timer('test_elapsed_nonstarted_fails')
        with pytest.raises(AssertionError):
            t.elapsed()

    def test_stop_nonstarted_fails(self):
        t = Timer('test_stop_nonstarted_fails')
        with pytest.raises(AssertionError):
            t.stop()

    def test_ncalls(self):
        t = Timer('test_ncalls')
        for i in range(10):
            t.start()
            t.stop()
        assert t.ncalls == 10

    def test_reset_timers(self):
        tic('test_reset')
        toc('test_reset')
        reset_timers()
        assert get_timers().keys() == []


if __name__ == '__main__':
    import os
    pytest.main(os.path.abspath(__file__))
